@@ -22,9 +22,10 @@ from . import (
     narrowspec,
     phases,
     pycompat,
-    repository,
     setdiscovery,
 )
+from .interfaces import repository
+
 
 def pull(pullop):
     """Pull using wire protocol version 2."""
@@ -38,7 +39,7 @@ def pull(pullop):
     # incremental pull. This is somewhat hacky and is not nearly robust enough
     # for long-term usage.
     if usingrawchangelogandmanifest:
-        with repo.transaction('clone'):
+        with repo.transaction(b'clone'):
             _fetchrawstorefiles(repo, remote)
             repo.invalidate(clearfilecache=True)
 
@@ -46,11 +47,13 @@ def pull(pullop):
 
     # We don't use the repo's narrow matcher here because the patterns passed
     # to exchange.pull() could be different.
-    narrowmatcher = narrowspec.match(repo.root,
-                                     # Empty maps to nevermatcher. So always
-                                     # set includes if missing.
-                                     pullop.includepats or {'path:.'},
-                                     pullop.excludepats)
+    narrowmatcher = narrowspec.match(
+        repo.root,
+        # Empty maps to nevermatcher. So always
+        # set includes if missing.
+        pullop.includepats or {b'path:.'},
+        pullop.excludepats,
+    )
 
     if pullop.includepats or pullop.excludepats:
         pathfilter = {}
@@ -63,7 +66,8 @@ def pull(pullop):
 
     # Figure out what needs to be fetched.
     common, fetch, remoteheads = _pullchangesetdiscovery(
-        repo, remote, pullop.heads, abortwhenunrelated=pullop.force)
+        repo, remote, pullop.heads, abortwhenunrelated=pullop.force
+    )
 
     # And fetch the data.
     pullheads = pullop.heads or remoteheads
@@ -74,23 +78,32 @@ def pull(pullop):
 
     # Ensure all new changesets are draft by default. If the repo is
     # publishing, the phase will be adjusted by the loop below.
-    if csetres['added']:
-        phases.registernew(repo, tr, phases.draft, csetres['added'])
+    if csetres[b'added']:
+        phases.registernew(repo, tr, phases.draft, csetres[b'added'])
 
     # And adjust the phase of all changesets accordingly.
     for phase in phases.phasenames:
-        if phase == b'secret' or not csetres['nodesbyphase'][phase]:
+        if phase == b'secret' or not csetres[b'nodesbyphase'][phase]:
             continue
 
-        phases.advanceboundary(repo, tr, phases.phasenames.index(phase),
-                               csetres['nodesbyphase'][phase])
+        phases.advanceboundary(
+            repo,
+            tr,
+            phases.phasenames.index(phase),
+            csetres[b'nodesbyphase'][phase],
+        )
 
     # Write bookmark updates.
-    bookmarks.updatefromremote(repo.ui, repo, csetres['bookmarks'],
-                               remote.url(), pullop.gettransaction,
-                               explicit=pullop.explicitbookmarks)
+    bookmarks.updatefromremote(
+        repo.ui,
+        repo,
+        csetres[b'bookmarks'],
+        remote.url(),
+        pullop.gettransaction,
+        explicit=pullop.explicitbookmarks,
+    )
 
-    manres = _fetchmanifests(repo, tr, remote, csetres['manifestnodes'])
+    manres = _fetchmanifests(repo, tr, remote, csetres[b'manifestnodes'])
 
     # We don't properly support shallow changeset and manifest yet. So we apply
     # depth limiting locally.
@@ -98,8 +111,9 @@ def pull(pullop):
         relevantcsetnodes = set()
         clnode = repo.changelog.node
 
-        for rev in repo.revs(b'ancestors(%ln, %s)',
-                             pullheads, pullop.depth - 1):
+        for rev in repo.revs(
+            b'ancestors(%ln, %s)', pullheads, pullop.depth - 1
+        ):
             relevantcsetnodes.add(clnode(rev))
 
         csetrelevantfilter = lambda n: n in relevantcsetnodes
@@ -128,15 +142,24 @@ def pull(pullop):
             manifestlinkrevs[mnode] = rev
 
     else:
-        csetsforfiles = [n for n in csetres['added'] if csetrelevantfilter(n)]
-        mnodesforfiles = manres['added']
-        manifestlinkrevs = manres['linkrevs']
+        csetsforfiles = [n for n in csetres[b'added'] if csetrelevantfilter(n)]
+        mnodesforfiles = manres[b'added']
+        manifestlinkrevs = manres[b'linkrevs']
 
     # Find all file nodes referenced by added manifests and fetch those
     # revisions.
     fnodes = _derivefilesfrommanifests(repo, narrowmatcher, mnodesforfiles)
-    _fetchfilesfromcsets(repo, tr, remote, pathfilter, fnodes, csetsforfiles,
-                         manifestlinkrevs, shallow=bool(pullop.depth))
+    _fetchfilesfromcsets(
+        repo,
+        tr,
+        remote,
+        pathfilter,
+        fnodes,
+        csetsforfiles,
+        manifestlinkrevs,
+        shallow=bool(pullop.depth),
+    )
+
 
 def _checkuserawstorefiledata(pullop):
     """Check whether we should use rawstorefiledata command to retrieve data."""
@@ -163,17 +186,19 @@ def _checkuserawstorefiledata(pullop):
 
     return True
 
+
 def _fetchrawstorefiles(repo, remote):
     with remote.commandexecutor() as e:
-        objs = e.callcommand(b'rawstorefiledata', {
-            b'files': [b'changelog', b'manifestlog'],
-        }).result()
+        objs = e.callcommand(
+            b'rawstorefiledata', {b'files': [b'changelog', b'manifestlog'],}
+        ).result()
 
         # First object is a summary of files data that follows.
         overall = next(objs)
 
-        progress = repo.ui.makeprogress(_('clone'), total=overall[b'totalsize'],
-                                        unit=_('bytes'))
+        progress = repo.ui.makeprogress(
+            _(b'clone'), total=overall[b'totalsize'], unit=_(b'bytes')
+        )
         with progress:
             progress.update(0)
 
@@ -186,14 +211,17 @@ def _fetchrawstorefiles(repo, remote):
 
                 for k in (b'location', b'path', b'size'):
                     if k not in filemeta:
-                        raise error.Abort(_(b'remote file data missing key: %s')
-                                          % k)
+                        raise error.Abort(
+                            _(b'remote file data missing key: %s') % k
+                        )
 
                 if filemeta[b'location'] == b'store':
                     vfs = repo.svfs
                 else:
-                    raise error.Abort(_(b'invalid location for raw file data: '
-                                        b'%s') % filemeta[b'location'])
+                    raise error.Abort(
+                        _(b'invalid location for raw file data: %s')
+                        % filemeta[b'location']
+                    )
 
                 bytesremaining = filemeta[b'size']
 
@@ -207,10 +235,13 @@ def _fetchrawstorefiles(repo, remote):
                         bytesremaining -= len(chunk)
 
                         if bytesremaining < 0:
-                            raise error.Abort(_(
-                                b'received invalid number of bytes for file '
-                                b'data; expected %d, got extra') %
-                                              filemeta[b'size'])
+                            raise error.Abort(
+                                _(
+                                    b'received invalid number of bytes for file '
+                                    b'data; expected %d, got extra'
+                                )
+                                % filemeta[b'size']
+                            )
 
                         progress.increment(step=len(chunk))
                         fh.write(chunk)
@@ -219,15 +250,25 @@ def _fetchrawstorefiles(repo, remote):
                             if chunk.islast:
                                 break
                         except AttributeError:
-                            raise error.Abort(_(
-                                b'did not receive indefinite length bytestring '
-                                b'for file data'))
+                            raise error.Abort(
+                                _(
+                                    b'did not receive indefinite length bytestring '
+                                    b'for file data'
+                                )
+                            )
 
                 if bytesremaining:
-                    raise error.Abort(_(b'received invalid number of bytes for'
-                                        b'file data; expected %d got %d') %
-                                      (filemeta[b'size'],
-                                       filemeta[b'size'] - bytesremaining))
+                    raise error.Abort(
+                        _(
+                            b'received invalid number of bytes for'
+                            b'file data; expected %d got %d'
+                        )
+                        % (
+                            filemeta[b'size'],
+                            filemeta[b'size'] - bytesremaining,
+                        )
+                    )
+
 
 def _pullchangesetdiscovery(repo, remote, heads, abortwhenunrelated=True):
     """Determine which changesets need to be pulled."""
@@ -240,7 +281,8 @@ def _pullchangesetdiscovery(repo, remote, heads, abortwhenunrelated=True):
     # TODO wire protocol version 2 is capable of more efficient discovery
     # than setdiscovery. Consider implementing something better.
     common, fetch, remoteheads = setdiscovery.findcommonheads(
-        repo.ui, repo, remote, abortwhenunrelated=abortwhenunrelated)
+        repo.ui, repo, remote, abortwhenunrelated=abortwhenunrelated
+    )
 
     common = set(common)
     remoteheads = set(remoteheads)
@@ -260,6 +302,7 @@ def _pullchangesetdiscovery(repo, remote, heads, abortwhenunrelated=True):
 
     return common, fetch, remoteheads
 
+
 def _fetchchangesets(repo, tr, remote, common, fetch, remoteheads):
     # TODO consider adding a step here where we obtain the DAG shape first
     # (or ask the server to slice changesets into chunks for us) so that
@@ -267,22 +310,27 @@ def _fetchchangesets(repo, tr, remote, common, fetch, remoteheads):
     # resuming interrupted clones, higher server-side cache hit rates due
     # to smaller segments, etc.
     with remote.commandexecutor() as e:
-        objs = e.callcommand(b'changesetdata', {
-            b'revisions': [{
-                b'type': b'changesetdagrange',
-                b'roots': sorted(common),
-                b'heads': sorted(remoteheads),
-            }],
-            b'fields': {b'bookmarks', b'parents', b'phase', b'revision'},
-        }).result()
+        objs = e.callcommand(
+            b'changesetdata',
+            {
+                b'revisions': [
+                    {
+                        b'type': b'changesetdagrange',
+                        b'roots': sorted(common),
+                        b'heads': sorted(remoteheads),
+                    }
+                ],
+                b'fields': {b'bookmarks', b'parents', b'phase', b'revision'},
+            },
+        ).result()
 
         # The context manager waits on all response data when exiting. So
         # we need to remain in the context manager in order to stream data.
         return _processchangesetdata(repo, tr, objs)
 
+
 def _processchangesetdata(repo, tr, objs):
-    repo.hook('prechangegroup', throw=True,
-              **pycompat.strkwargs(tr.hookargs))
+    repo.hook(b'prechangegroup', throw=True, **pycompat.strkwargs(tr.hookargs))
 
     urepo = repo.unfiltered()
     cl = urepo.changelog
@@ -293,14 +341,14 @@ def _processchangesetdata(repo, tr, objs):
     # follows.
     meta = next(objs)
 
-    progress = repo.ui.makeprogress(_('changesets'),
-                                    unit=_('chunks'),
-                                    total=meta.get(b'totalitems'))
+    progress = repo.ui.makeprogress(
+        _(b'changesets'), unit=_(b'chunks'), total=meta.get(b'totalitems')
+    )
 
     manifestnodes = {}
 
     def linkrev(node):
-        repo.ui.debug('add changeset %s\n' % short(node))
+        repo.ui.debug(b'add changeset %s\n' % short(node))
         # Linkrev for changelog is always self.
         return len(cl)
 
@@ -358,18 +406,20 @@ def _processchangesetdata(repo, tr, objs):
                 0,
             )
 
-    added = cl.addgroup(iterrevisions(), linkrev, weakref.proxy(tr),
-                        addrevisioncb=onchangeset)
+    added = cl.addgroup(
+        iterrevisions(), linkrev, weakref.proxy(tr), addrevisioncb=onchangeset
+    )
 
     progress.complete()
 
     return {
-        'added': added,
-        'nodesbyphase': nodesbyphase,
-        'bookmarks': remotebookmarks,
-        'manifestnodes': manifestnodes,
+        b'added': added,
+        b'nodesbyphase': nodesbyphase,
+        b'bookmarks': remotebookmarks,
+        b'manifestnodes': manifestnodes,
     }
 
+
 def _fetchmanifests(repo, tr, remote, manifestnodes):
     rootmanifest = repo.manifestlog.getstorage(b'')
 
@@ -379,7 +429,7 @@ def _fetchmanifests(repo, tr, remote, manifestnodes):
     linkrevs = {}
     seen = set()
 
-    for clrev, node in sorted(manifestnodes.iteritems()):
+    for clrev, node in sorted(pycompat.iteritems(manifestnodes)):
         if node in seen:
             continue
 
@@ -427,13 +477,14 @@ def _fetchmanifests(repo, tr, remote, manifestnodes):
                 basenode,
                 delta,
                 # Flags not yet supported.
-                0
+                0,
             )
 
             progress.increment()
 
-    progress = repo.ui.makeprogress(_('manifests'), unit=_('chunks'),
-                                    total=len(fetchnodes))
+    progress = repo.ui.makeprogress(
+        _(b'manifests'), unit=_(b'chunks'), total=len(fetchnodes)
+    )
 
     commandmeta = remote.apidescriptor[b'commands'][b'manifestdata']
     batchsize = commandmeta.get(b'recommendedbatchsize', 10000)
@@ -450,33 +501,40 @@ def _fetchmanifests(repo, tr, remote, manifestnodes):
     added = []
 
     for i in pycompat.xrange(0, len(fetchnodes), batchsize):
-        batch = [node for node in fetchnodes[i:i + batchsize]]
+        batch = [node for node in fetchnodes[i : i + batchsize]]
         if not batch:
             continue
 
         with remote.commandexecutor() as e:
-            objs = e.callcommand(b'manifestdata', {
-                b'tree': b'',
-                b'nodes': batch,
-                b'fields': {b'parents', b'revision'},
-                b'haveparents': True,
-            }).result()
+            objs = e.callcommand(
+                b'manifestdata',
+                {
+                    b'tree': b'',
+                    b'nodes': batch,
+                    b'fields': {b'parents', b'revision'},
+                    b'haveparents': True,
+                },
+            ).result()
 
             # Chomp off header object.
             next(objs)
 
-            added.extend(rootmanifest.addgroup(
-                iterrevisions(objs, progress),
-                linkrevs.__getitem__,
-                weakref.proxy(tr)))
+            added.extend(
+                rootmanifest.addgroup(
+                    iterrevisions(objs, progress),
+                    linkrevs.__getitem__,
+                    weakref.proxy(tr),
+                )
+            )
 
     progress.complete()
 
     return {
-        'added': added,
-        'linkrevs': linkrevs,
+        b'added': added,
+        b'linkrevs': linkrevs,
     }
 
+
 def _derivefilesfrommanifests(repo, matcher, manifestnodes):
     """Determine what file nodes are relevant given a set of manifest nodes.
 
@@ -487,7 +545,8 @@ def _derivefilesfrommanifests(repo, matcher, manifestnodes):
     fnodes = collections.defaultdict(dict)
 
     progress = repo.ui.makeprogress(
-        _('scanning manifests'), total=len(manifestnodes))
+        _(b'scanning manifests'), total=len(manifestnodes)
+    )
 
     with progress:
         for manifestnode in manifestnodes:
@@ -509,8 +568,10 @@ def _derivefilesfrommanifests(repo, matcher, manifestnodes):
 
     return fnodes
 
+
 def _fetchfiles(repo, tr, remote, fnodes, linkrevs):
     """Fetch file data from explicit file revisions."""
+
     def iterrevisions(objs, progress):
         for filerevision in objs:
             node = filerevision[b'node']
@@ -544,15 +605,17 @@ def _fetchfiles(repo, tr, remote, fnodes, linkrevs):
             progress.increment()
 
     progress = repo.ui.makeprogress(
-        _('files'), unit=_('chunks'),
-         total=sum(len(v) for v in fnodes.itervalues()))
+        _(b'files'),
+        unit=_(b'chunks'),
+        total=sum(len(v) for v in pycompat.itervalues(fnodes)),
+    )
 
     # TODO make batch size configurable
     batchsize = 10000
     fnodeslist = [x for x in sorted(fnodes.items())]
 
     for i in pycompat.xrange(0, len(fnodeslist), batchsize):
-        batch = [x for x in fnodeslist[i:i + batchsize]]
+        batch = [x for x in fnodeslist[i : i + batchsize]]
         if not batch:
             continue
 
@@ -561,16 +624,25 @@ def _fetchfiles(repo, tr, remote, fnodes, linkrevs):
             locallinkrevs = {}
 
             for path, nodes in batch:
-                fs.append((path, e.callcommand(b'filedata', {
-                    b'path': path,
-                    b'nodes': sorted(nodes),
-                    b'fields': {b'parents', b'revision'},
-                    b'haveparents': True,
-                })))
+                fs.append(
+                    (
+                        path,
+                        e.callcommand(
+                            b'filedata',
+                            {
+                                b'path': path,
+                                b'nodes': sorted(nodes),
+                                b'fields': {b'parents', b'revision'},
+                                b'haveparents': True,
+                            },
+                        ),
+                    )
+                )
 
                 locallinkrevs[path] = {
                     node: linkrevs[manifestnode]
-                    for node, manifestnode in nodes.iteritems()}
+                    for node, manifestnode in pycompat.iteritems(nodes)
+                }
 
             for path, f in fs:
                 objs = f.result()
@@ -582,10 +654,13 @@ def _fetchfiles(repo, tr, remote, fnodes, linkrevs):
                 store.addgroup(
                     iterrevisions(objs, progress),
                     locallinkrevs[path].__getitem__,
-                    weakref.proxy(tr))
+                    weakref.proxy(tr),
+                )
 
-def _fetchfilesfromcsets(repo, tr, remote, pathfilter, fnodes, csets,
-                         manlinkrevs, shallow=False):
+
+def _fetchfilesfromcsets(
+    repo, tr, remote, pathfilter, fnodes, csets, manlinkrevs, shallow=False
+):
     """Fetch file data from explicit changeset revisions."""
 
     def iterrevisions(objs, remaining, progress):
@@ -629,8 +704,10 @@ def _fetchfilesfromcsets(repo, tr, remote, pathfilter, fnodes, csets,
             remaining -= 1
 
     progress = repo.ui.makeprogress(
-        _('files'), unit=_('chunks'),
-        total=sum(len(v) for v in fnodes.itervalues()))
+        _(b'files'),
+        unit=_(b'chunks'),
+        total=sum(len(v) for v in pycompat.itervalues(fnodes)),
+    )
 
     commandmeta = remote.apidescriptor[b'commands'][b'filesdata']
     batchsize = commandmeta.get(b'recommendedbatchsize', 50000)
@@ -651,16 +728,15 @@ def _fetchfilesfromcsets(repo, tr, remote, pathfilter, fnodes, csets,
         fields.add(b'linknode')
 
     for i in pycompat.xrange(0, len(csets), batchsize):
-        batch = [x for x in csets[i:i + batchsize]]
+        batch = [x for x in csets[i : i + batchsize]]
         if not batch:
             continue
 
         with remote.commandexecutor() as e:
             args = {
-                b'revisions': [{
-                    b'type': b'changesetexplicit',
-                    b'nodes': batch,
-                }],
+                b'revisions': [
+                    {b'type': b'changesetexplicit', b'nodes': batch,}
+                ],
                 b'fields': fields,
                 b'haveparents': haveparents,
             }
@@ -682,7 +758,8 @@ def _fetchfilesfromcsets(repo, tr, remote, pathfilter, fnodes, csets,
 
                 linkrevs = {
                     fnode: manlinkrevs[mnode]
-                    for fnode, mnode in fnodes[path].iteritems()}
+                    for fnode, mnode in pycompat.iteritems(fnodes[path])
+                }
 
                 def getlinkrev(node):
                     if node in linkrevs:
@@ -690,8 +767,9 @@ def _fetchfilesfromcsets(repo, tr, remote, pathfilter, fnodes, csets,
                     else:
                         return clrev(node)
 
-                store.addgroup(iterrevisions(objs, header[b'totalitems'],
-                                             progress),
-                               getlinkrev,
-                               weakref.proxy(tr),
-                               maybemissingparents=shallow)
+                store.addgroup(
+                    iterrevisions(objs, header[b'totalitems'], progress),
+                    getlinkrev,
+                    weakref.proxy(tr),
+                    maybemissingparents=shallow,
+                )
